﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Foundation;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;

namespace TimelinePanel
{
    public class TimelinePanel : Panel
    {
        public static readonly DependencyProperty MinTimeProperty =
            DependencyProperty.Register("MinTime", typeof(DateTime), typeof(TimelinePanel), new PropertyMetadata(DateTime.Parse("07:00")));

        public static readonly DependencyProperty MaxTimeProperty =
              DependencyProperty.Register("MaxTime", typeof(DateTime), typeof(TimelinePanel), new PropertyMetadata(DateTime.Parse("15:00")));
        
        public DateTime MinTime
        {
            get { return (DateTime)GetValue(MinTimeProperty); }
            set { SetValue(MinTimeProperty, value); }
        }

        public DateTime MaxTime
        {
            get { return (DateTime)GetValue(MaxTimeProperty); }
            set { SetValue(MaxTimeProperty, value); }
        }
        
        public static readonly DependencyProperty StartTimeProperty =
          DependencyProperty.RegisterAttached("StartTime", typeof(DateTime), typeof(TimelinePanel), new PropertyMetadata(DateTime.MinValue));

        public static readonly DependencyProperty EndTimeProperty =
            DependencyProperty.RegisterAttached("EndTime", typeof(DateTime), typeof(TimelinePanel), new PropertyMetadata(DateTime.MaxValue));

        public static DateTime GetStartTime(DependencyObject obj)
        {
            return (DateTime)obj.GetValue(StartTimeProperty);
        }

        public static void SetStartTime(DependencyObject obj, DateTime value)
        {
            obj.SetValue(StartTimeProperty, value);
        }

        public static DateTime GetEndTime(DependencyObject obj)
        {
            return (DateTime)obj.GetValue(EndTimeProperty);
        }

        public static void SetEndTime(DependencyObject obj, DateTime value)
        {
            obj.SetValue(EndTimeProperty, value);
        }

        //protected override Size MeasureOverride(Size availableSize)
        //{
        //    Size desiredSize = new Size();
        //    var maxHeight = 0.0;
        //    foreach (UIElement child in Children)
        //    {
        //        child.Measure(availableSize);
        //        if (child.DesiredSize.Height > maxHeight)
        //        {
        //            maxHeight = child.DesiredSize.Height;
        //        }
        //    }

        //    desiredSize.Height = maxHeight;
        //    // desiredSize = ... computed sum of children's DesiredSize ...; 
        //    // IMPORTANT: do not allow PositiveInfinity to be returned, that will raise an exception in the caller! 
        //    // PositiveInfinity might be an availableSize input; this means that the parent does not care about sizing 
        //    return desiredSize;
        //}

        protected override Size ArrangeOverride(Size finalSize)
        {
            double range = (MaxTime - MinTime).Ticks;
            double pixelsPerTick = finalSize.Width / range;

            foreach (UIElement child in Children)
            {
                ArrangeChild(child, pixelsPerTick, finalSize.Height);
            }

            return finalSize;
        }

        private void ArrangeChild(UIElement child, double pixelsPerTick, double elementHeight)
        {
            DateTime childStartDate = GetStartTime(child);
            DateTime childEndDate = GetEndTime(child);

            TimeSpan childDuration = childEndDate - childStartDate;

            double offset;
            double width;
            if (childDuration.Ticks >= 0)
            {
                offset = (childStartDate - MinTime).Ticks * pixelsPerTick;
                width = childDuration.Ticks * pixelsPerTick;
            }
            else
            {
                offset = 0;
                width = 0;
            }

            child.Arrange(new Rect(offset, 0, width, elementHeight));
        }


    }
}
